@@ -1,7 +1,7 @@
 /*
  * common routines for editing / adding accounts
  *
- * Copyright (C) 2014-2016 LastPass.
+ * Copyright (C) 2014-2018 LastPass.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,8 @@
 #include <errno.h>
 #include "util.h"
 
+#define MAX_NOTE_LEN (unsigned long) 45000
+
 #if defined(__linux__) || defined(__CYGWIN__)
 static char *shared_memory_dir(void)
 {
@@ -131,29 +133,41 @@ _noreturn_ static inline void die_unlink_errno(const char *str, const char *file
 static void assign_account_value(struct account *account,
 				 const char *label,
 				 char *value,
+				 int lineno,
 				 unsigned char key[KDF_HASH_LEN])
 {
 	struct field *editable_field = NULL;
 
 #define assign_if(title, field) do { \
 	if (!strcmp(label, title)) { \
-		account_set_##field(account, value, key); \
+		account_set_##field(account, xstrdup(trim(value)), key); \
 		return; \
 	} \
 	} while (0)
 
-	value = xstrdup(trim(value));
-
-	assign_if("Name", fullname);
+	/*
+	 * "Name" may be used in note templates; only assign fullname
+	 * in the first line.
+	 */
+	if (lineno == 1) {
+		assign_if("Name", fullname);
+	}
 	assign_if("URL", url);
 	assign_if("Username", username);
 	assign_if("Password", password);
 	assign_if("Application", appname);
+	assign_if("Notes", note);
+
+	if (!strcmp(label, "Reprompt")) {
+		account->pwprotect = !strcmp(trim(value), "Yes");
+		return;
+	}
 
 	/* if we got here maybe it's a secure note field */
 	list_for_each_entry(editable_field, &account->field_head, list) {
 		if (!strcmp(label, editable_field->name)) {
-			field_set_value(account, editable_field, value, key);
+			field_set_value(account, editable_field,
+					xstrdup(trim(value)), key);
 			return;
 		}
 	}
@@ -162,7 +176,7 @@ static void assign_account_value(struct account *account,
 	editable_field = new0(struct field, 1);
 	editable_field->name = xstrdup(label);
 	editable_field->type = xstrdup("password");
-	field_set_value(account, editable_field, value, key);
+	field_set_value(account, editable_field, xstrdup(trim(value)), key);
 	list_add_tail(&editable_field->list, &account->field_head);
 
 #undef assign_if
@@ -193,45 +207,100 @@ int read_file_buf(FILE *fp, char **value_out, size_t *len_out)
 	return 0;
 }
 
+enum note_type get_note_type(struct account *account)
+{
+	struct field *editable_field;
+
+	list_for_each_entry(editable_field, &account->field_head, list) {
+		if (!strcmp(editable_field->name, "NoteType")) {
+			return notes_get_type_by_name(editable_field->value);
+		}
+	}
+	return NOTE_TYPE_NONE;
+}
+
+struct parsed_name_value
+{
+	char *name;
+	char *value;
+	int lineno;
+	struct list_head list;
+};
+
 /*
  * Read a file representing all of the data in an account.
  * We generate this file when editing an account, and parse it back
- * after a user has edited it.  Each line, with the exception of the
- * final "notes" label, is parsed from the end of the label to the
- * first newline.  In the case of notes, the rest of the file is considered
- * part of the note.
+ * after a user has edited it.
+ *
+ * Multiline values are accepted (though they may not be supported by
+ * lastpass in all cases).
+ *
+ * Once the "Notes:" label is encountered, everything else is concatenated
+ * into the note.
  *
  * Name: text0
  * URL: text1
  * [...]
  * Notes:
  * notes text here
- *
  */
-static void parse_account_file(FILE *input, struct account *account,
-			       unsigned char key[KDF_HASH_LEN])
+static void parse_account_file(FILE *input, enum note_type note_type,
+			       struct list_head *list_head)
 {
 	_cleanup_free_ char *line = NULL;
 	ssize_t read;
 	size_t len = 0;
-	char *label, *delim, *value;
+	char *name, *delim, *value = NULL;
 	bool parsing_notes = false;
 	int ret;
+	int lineno = 0;
+
+	struct parsed_name_value *current = NULL;
 
 	/* parse label: [value] */
 	while ((read = getline(&line, &len, input)) != -1) {
-		delim = strchr(line, ':');
-		if (!delim)
-			continue;
-		*delim = 0;
-		value = delim + 1;
-		label = line;
+		lineno++;
 
-		if (!strcmp(label, "Notes")) {
+		line = trim(line);
+		delim = strchr(line, ':');
+		if (!delim) {
+			/* non keyed strings go to existing field (if any) */
+			if (current)
+				xstrappendf(&current->value, "\n%s", line);
+			continue;
+		}
+
+		name = xstrndup(line, delim - line);
+		value = xstrdup(delim + 1);
+
+		/*
+		 * If this is a known notetype, append any non-existent
+		 * keys to the existing field.  For example, Proc-Type
+		 * in the ssh private key field goes into private key,
+		 * not a Proc-Type field.
+		 */
+		if (note_type != NOTE_TYPE_NONE &&
+		    !note_has_field(note_type, name) && current &&
+		    note_field_is_multiline(note_type, current->name)) {
+			xstrappendf(&current->value, "\n%s", line);
+
+			free(name);
+			free(value);
+			continue;
+		}
+
+		if (!strcmp(name, "Notes")) {
 			parsing_notes = true;
+			free(name);
+			free(value);
 			break;
 		}
-		assign_account_value(account, label, value, key);
+
+		current = new0(struct parsed_name_value, 1);
+		current->name = name;
+		current->value = value;
+		current->lineno = lineno;
+		list_add_tail(&current->list, list_head);
 	}
 
 	if (!parsing_notes)
@@ -244,7 +313,34 @@ static void parse_account_file(FILE *input, struct account *account,
 	if (ret)
 		return;
 
-	account_set_note(account, value, key);
+	if (len > MAX_NOTE_LEN) {
+		die("Maximum note length is %lu bytes (was %lu)",
+		    MAX_NOTE_LEN, len);
+	}
+
+	current = new0(struct parsed_name_value, 1);
+	current->name = xstrdup("Notes");
+	current->value = value;
+	current->lineno = lineno;
+	list_add_tail(&current->list, list_head);
+}
+
+static void read_account_file(FILE *input, struct account *account,
+			      unsigned char key[KDF_HASH_LEN])
+{
+	LIST_HEAD(fields);
+	struct parsed_name_value *entry, *tmp;
+
+	parse_account_file(input, get_note_type(account), &fields);
+
+	list_for_each_entry_safe(entry, tmp, &fields, list) {
+		assign_account_value(account, entry->name, entry->value,
+				     entry->lineno, key);
+		free(entry->name);
+		free(entry->value);
+		list_del(&entry->list);
+		free(entry);
+	}
 }
 
 static
@@ -288,20 +384,18 @@ static void add_default_fields(struct account *account,
 	 * does not already exist in the account.
 	 */
 	tmpl = &note_templates[note_type];
-	for (i=0; tmpl->fields[i]; i++)
+	for (i=0; tmpl->fields[i]; i++) {
+		/*
+		 * ... but skip these: they are already handled by the
+		 * collapse code.
+		 */
+		if (!strcmp(tmpl->fields[i], "Username"))
+			continue;
+		if (!strcmp(tmpl->fields[i], "Password"))
+			continue;
+
 		add_default_field(account, tmpl->fields[i], key);
-}
-
-enum note_type get_note_type(struct account *account)
-{
-	struct field *editable_field;
-
-	list_for_each_entry(editable_field, &account->field_head, list) {
-		if (!strcmp(editable_field->name, "NoteType")) {
-			return notes_get_type_by_name(editable_field->value);
-		}
 	}
-	return NOTE_TYPE_NONE;
 }
 
 static int write_account_file(FILE *fp, struct account *account,
@@ -335,6 +429,10 @@ static int write_account_file(FILE *fp, struct account *account,
 		write_field(editable_field->name, editable_field->value);
 	}
 
+	if (account->pwprotect) {
+		write_field("Reprompt", "Yes");
+	}
+
 	if (fprintf(fp, "Notes:    # Add notes below this line.\n%s", account->note) < 0)
 		return -errno;
 
@@ -429,9 +527,9 @@ int edit_account(struct session *session,
 		}
 		fclose(tmpfile);
 
-		xasprintf(&editcmd, "${EDITOR:-vi} '%s'", tmppath);
+		xasprintf(&editcmd, "${VISUAL:-${EDITOR:-vi}} '%s'", tmppath);
 		if (system(editcmd) < 0)
-			die_unlink_errno("system($EDITOR)", tmppath, tmpdir);
+			die_unlink_errno("system($VISUAL)", tmppath, tmpdir);
 
 		tmpfile = fopen(tmppath, "r");
 	} else
@@ -444,13 +542,12 @@ int edit_account(struct session *session,
 		if (ret)
 			die_unlink_errno("fread(tmpfile)", tmppath, tmpdir);
 	} else if (choice == EDIT_ANY) {
-		parse_account_file(tmpfile, editable, key);
+		read_account_file(tmpfile, editable, key);
 		value = NULL;
 	} else {
-		value = NULL;
-		len = 0;
-		if (getline(&value, &len, tmpfile) < 0)
-			die_unlink_errno("getline", tmppath, tmpdir);
+		ret = read_file_buf(tmpfile, &value, &len);
+		if (ret)
+			die_unlink_errno("fread(tmpfile)", tmppath, tmpdir);
 	}
 	fclose(tmpfile);
 
@@ -487,8 +584,8 @@ int edit_account(struct session *session,
 		notes_collapsed = notes_collapse(notes_expansion);
 		account_free(notes_expansion);
 		account_set_note(editable, xstrdup(notes_collapsed->note), key);
-		if (choice == EDIT_NAME)
-			account_set_fullname(editable, xstrdup(notes_collapsed->fullname), key);
+		account_set_fullname(editable, xstrdup(notes_collapsed->fullname), key);
+		editable->pwprotect = notes_collapsed->pwprotect;
 		account_free(notes_collapsed);
 	}
 
@@ -532,6 +629,8 @@ int edit_new_account(struct session *session,
 	}
 
 	account->id = xstrdup("0");
+	account->attachkey = xstrdup("");
+	account->attachkey_encrypted = xstrdup("");
 	account_set_password(account, xstrdup(""), key);
 	account_set_fullname(account, xstrdup(name), key);
 	account_set_username(account, xstrdup(""), key);
