@@ -5,8 +5,12 @@ const char *PK_PINS[] = {
 	"HXXQgxueCIU5TTLHob/bPbwcKOKw6DkfsTWYHbxbqTY=",
 	/* current lastpass.eu primary (AddTrust) */
 	"lCppFqbkrlJ3EcVFAkeip0+44VaoJUymbnOaEUk7tEU=",
+	/* future lastpass root CA (GlobalSign R1) */
+	"K87oWBWM9UZfyddvDfoxL+8lpNyoUB2ptGtn0fv6G2Q=",
 	/* future lastpass root CA (GlobalSign R2) */
 	"iie1VXtL7HzAMF+/PVPR9xzT80kQxdZeJ+zduCB3uj0=",
+	/* future lastpass root CA (GlobalSign R3) */
+	"cGuxAXyFXFkWm61cF4HPWX8S0srS9j0aSqN0k4AP+4A=",
 	/* future lastpass.com primary (leaf) */
 	"0hkr5YW/WE6Nq5hNTcApxpuaiwlwy5HUFiOt3Qd9VBc=",
 	/* future lastpass.com backup (leaf) */
