@@ -23,7 +23,7 @@ struct session {
 struct session *session_new();
 void session_free(struct session *session);
 bool session_is_valid(struct session *session);
-struct session *sesssion_load(unsigned const char key[KDF_HASH_LEN]);
+struct session *session_load(unsigned const char key[KDF_HASH_LEN]);
 void session_save(struct session *session, unsigned const char key[KDF_HASH_LEN]);
 void session_set_private_key(struct session *session, unsigned const char key[KDF_HASH_LEN], const char *key_hex);
 void session_kill();
